/*
	Paul Manta, 333CB
	2014
 */

#include <avr/interrupt.h>
#include <avr/io.h>


/* This value is used to control the frequency of the clock, which
   in turn determines the speed of the motors. It should be a 16 bit
   value (that is four hex digits). The larger this value is, the
   slower the motors. */
#define TIMER1_COMPARE_VALUE   0x0700u

#define DDR_BUTTON             DDRC
#define PORT_BUTTON            PORTC
#define PIN_BUTTON             PINC
#define BUTTON0                PC0

#define DDR_LED                DDRD
#define PORT_LED               PORTD
#define LED0                   PD7

#define DDR_PROXIM             DDRC
#define PORT_PROXIM            PORTC
#define PIN_PROXIM             PINC
#define PCMSK_PROXIM           PCMSK2
#define PCIE_PROXIM            PCIE2
#define PCINT_PROXIM           PCINT16
#define PCINT_PROXIM_vect      PCINT2_vect
#define PROXIM0                PC2

#define DDR_WHEEL0             DDRC
#define PORT_WHEEL0            PORTC
#define WHEEL0_ENB             PC1
#define WHEEL0_FWD             PC3
#define WHEEL0_REV             PC5

#define DDR_WHEEL1             DDRD
#define PORT_WHEEL1            PORTD
#define WHEEL1_ENB             PD1
#define WHEEL1_FWD             PD3
#define WHEEL1_REV             PD5

#define B(N)                   (1u << (N))


static int g_obstacle_ahead = 0;


ISR(PCINT_PROXIM_vect)
{
    g_obstacle_ahead = !g_obstacle_ahead;
}

ISR(TIMER1_COMPA_vect)
{
    if (g_obstacle_ahead) {
		PORT_WHEEL0 ^= B(WHEEL1_FWD);
		PORT_WHEEL0 &= ~B(WHEEL0_REV);

		PORT_WHEEL1 ^= B(WHEEL1_REV);
		PORT_WHEEL1 &= ~B(WHEEL1_FWD);
    }
    else {
        PORT_WHEEL0 ^= B(WHEEL0_FWD);
		PORT_WHEEL0 &= ~B(WHEEL0_REV);

		PORT_WHEEL1 ^= B(WHEEL1_FWD);
		PORT_WHEEL1 &= ~B(WHEEL1_REV);

    }
}

int main(void)
{
    /* Button */
    DDR_BUTTON   &= ~B(BUTTON0);                       /* Set as input */
    PORT_BUTTON  |= B(BUTTON0);                        /* Pull-up */

    /* LED */
    DDR_LED      |= B(LED0);                           /* Set as output */
    PORT_LED     &= ~B(LED0);                          /* Initially off */

    /* Proximity sensor */
    DDR_PROXIM   &= ~B(PROXIM0);                       /* Set as input */
    PORT_PROXIM  |= B(PROXIM0);                        /* Pull-up */
    PCICR        |= B(PCIE_PROXIM);                    /* Enable interrupt */
    PCMSK_PROXIM |= B(PCINT_PROXIM);

    /* Wheel0 */
    DDR_WHEEL0   |= B(WHEEL0_FWD) | B(WHEEL0_REV);     /* Output to WHEEL0 */
	PORT_WHEEL0  |= B(WHEEL0_ENB);                     /* Enable WHEEL0 */
	PORT_WHEEL0  &= ~(B(WHEEL0_FWD) | B(WHEEL0_REV));  /* Initially not rotating */

	/* Wheel1 */
	DDR_WHEEL1   |= B(WHEEL1_FWD) | B(WHEEL1_REV);     /* Output to WHEEL1 */
	PORT_WHEEL1  |= B(WHEEL1_ENB);                     /* Enable WHEEL1 */
	PORT_WHEEL1  &= ~(B(WHEEL1_FWD) | B(WHEEL1_REV));  /* Initially not rotating */

    /* Timer1 */
    TCCR1B |= B(CS12) | B(CS10); TCCR1B &= ~B(CS11);   /* Prescaler at 1024 */
    TCCR1B |= B(WGM12);                                /* Clear on compare */
    TIMSK1 |= B(OCIE1A);                               /* COMPA interrupt */
    OCR1A = TIMER1_COMPARE_VALUE;

    SREG |= B(7);
    sei();

    while (1) {
        /* Interrupts handle everything. */
    }

    return 0;
}
